#include <stdio.h>
#include <math.h>

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2registers.h>

#define PI 3.1415926535897932384626433832795
#define DEG2RAD(a) a*(PI/180)

#define SCRWIDTH 320
#define SCRHEIGHT 240

int PRIM_POINTS = 0;
int PRIM_TRIANGLE = 3;
int PRIM_TRI_STRIP = 4;
int PRIM_GOURAUD_ON = 8;
int PRIM_ABE = 64;

// Counter of how many vertices to draw within the next GIF packet.
int iVertexCounter;

// aVertData - is used for vertices and aRGBAData is used for colors 
typedef int aVertData[3];
typedef int aRGBAData[4];

void waitForGIF();

void constructGIF(int PRIM_TYPE, aVertData *aVertices, aRGBAData *aColors){
 
  sps2GIFTag_t gifTag;
  sps2GIFPackedRegister_t gifRegister;
  int iPointLooper;

  // Prepare the GIF tag  
  gifTag.i128=0;       // Blank it out
  gifTag.s.NLOOP=iVertexCounter;
  gifTag.s.PRE=1;      // We are providing a valid PRIM value
  gifTag.s.EOP=1;      // End of packet -- draw me please!
  gifTag.s.PRIM=PRIM_TYPE;
  gifTag.s.FLG=GIF_FLG_PACKED; // We're using the packed format for data
  gifTag.s.NREG=0;     // Start off with zero registers, increment as we
  // populate the descriptions
  
  // Add a register.  It will be an RGBAQ register
  SPS2_SET_GIF_REG(gifTag, gifTag.s.NREG++, GIF_REG_RGBAQ);
  // Add another register.  It will be an XYZ2 register
  SPS2_SET_GIF_REG(gifTag, gifTag.s.NREG++, GIF_REG_XYZ2);

  waitForGIF();
  // Send the GIF tag
  DPUT_EE_GIF_FIFO(gifTag.i128);

  for (iPointLooper=0; iPointLooper<iVertexCounter;iPointLooper++) {
    // First, prepare and sent the RGBAQ register
    gifRegister.i128=0;
    gifRegister.RGBAQ.s.R=aColors[iPointLooper][0];
    gifRegister.RGBAQ.s.G=aColors[iPointLooper][1];
    gifRegister.RGBAQ.s.B=aColors[iPointLooper][2];
    gifRegister.RGBAQ.s.A=aColors[iPointLooper][3];

    waitForGIF();
    DPUT_EE_GIF_FIFO(gifRegister.i128);

    // Next, prepare and send the XYZ2 register
    gifRegister.i128=0;
    gifRegister.XYZ2.s.X=(aVertices[iPointLooper][0]+ (2048) ) << 4;
    gifRegister.XYZ2.s.Y=(aVertices[iPointLooper][1]+ (2048) ) << 4;
    gifRegister.XYZ2.s.Z=(aVertices[iPointLooper][2]+ (2048) ) << 4;

    waitForGIF();
    DPUT_EE_GIF_FIFO(gifRegister.i128);

   }
}

void waitForGIF(){
  // Check to see if FIFO is empty
  while ( ((*EE_GIF_STAT)&(0x10<<24)) != 0);
}













