/*
  SPS2-Pong 0.1
  By David Springate <djspringate@<removethis>mail.ru>
  and Steven Craft

  <REQUIRES SPS2 LIBS>

  Thanks to Morten 'Sparky' Mikkelsen, Steven 'Sauce' Osman and Lee 'Idiot' Bamber.

  Started 11/01/04

  Note: When dealing with the ball - 0 degrees is down and 90 degrees is right.

  CHANGES NEEDED: Change the paddles to include an X; Vibrate pad when ball hits player's paddle

*/

#include "engine/gif.c"
#include "engine/joypadlib.h"

void drawCuboid(float x, float y, float z, float width, float height, float depth, aRGBAData *colors);
void drawBall();
void drawBackground();
void drawPaddle();

typedef struct BALL{
  float x; float y; float angle; float speed; float size; } ball;

typedef struct PADDLE{
  float y; float speed; int width; int height; } paddle;

ball myBall;
paddle myPaddle[2];

unsigned long buttons;
unsigned char button_pressures[PS2PAD_NUM_PRESSURE_BUTTONS];
float left_stick[2], right_stick[2];
int active_joypad = 0;

int main() {
  int i;

  int iSPS2Descriptor;

  iSPS2Descriptor=sps2Init();

  sps2UScreenInit(0);

  iVertexCounter = 0;
  myBall.x=0;
  myBall.y=20;
  myBall.angle=DEG2RAD(45);
  myBall.speed=4;
  myBall.size = 10;

  for (i = 0; i < 2; i++){
    myPaddle[i].y = 0;
    myPaddle[i].width = 10;
    myPaddle[i].height = 50;
    myPaddle[i].speed = 2;
  }

  joypadlib_initialize();

  while (1) {

    sps2UScreenClear(0,0,0);

    joypadlib_update();

    while (active_joypad < 2){

    buttons = joypadlib_get_joypad_buttons(active_joypad, button_pressures, PRESSURE_TYPE_BYTE);
    joypadlib_get_joypad_sticks(active_joypad, left_stick, right_stick);

    if (buttons & PS2PAD_BUTTON_START)
      return 0;

    if (buttons & PS2PAD_BUTTON_UP)
      myPaddle[active_joypad].y-=myPaddle[active_joypad].speed;

    if (buttons & PS2PAD_BUTTON_DOWN)
      myPaddle[active_joypad].y+=myPaddle[active_joypad].speed;

    active_joypad++;

    }

    active_joypad=0;

    // Test Cube
    //drawCuboid(0,0,0,50,-50,-50, NULL);

    // Draw background
    drawBackground();

    // Draw ball
    drawBall();

    // Draw paddles
    drawPaddle();

    sps2UScreenSwap();
  }

  sps2UScreenShutdown();

  sps2Release(iSPS2Descriptor);
  return 0;
}

void drawCuboid(float x, float y, float z, float width, float height, float depth, aRGBAData *colors) {

  aVertData aVertices[]={
    // 24 vertices

    //Front
    {   x+width,  y, z},
    {  x,  y, z},
    {  x+width, y+height, z},
    {  x, y+height, z},
    
    //Back
    {  x+width,  y, z+depth},
    {  x,  y, z+depth},
    {  x+width, y+height, z+depth},
    {  x, y+height, z+depth},

    //Left
    {   x,  y, z+depth},
    {  x,  y, z},
    {  x, y+height, z+depth},
    {  x, y+height, z},

    //Right
    {  x+width,  y, z+depth},
    {  x+width,  y, z},
    {  x+width, y+height, z+depth},
    {  x+width, y+height, z},

    //Top
    {  x+width, y+height, z},
    {  x,       y+height, z},
    {  x+width, y+height, z+depth},
    {  x,       y+height, z+depth},
    
    //Bottom
    {  x+width, y, z},
    {  x,       y, z},
    {  x+width, y, z+depth},
    {  x,       y, z+depth}
    
  };

  aRGBAData aDefaultColors[]={
    {255,  0,  0,  0},
    {  0,255,  0,255},
    {  0,  0,255,  0},
    {255,255,255,  0}
  };

  if (colors == NULL)
    colors=aDefaultColors;

  iVertexCounter=24;
  constructGIF(PRIM_TRI_STRIP+PRIM_GOURAUD_ON, aVertices, colors);
}

void drawBall(){
   
  aRGBAData ballColor[]={
    { 255, 255, 255, 0 },
    { 255, 255, 255, 0 },
    { 255, 255, 255, 0 },
    { 255, 255, 255, 0 }
  };

  // Wall collisions

  if (myBall.y > (SCRHEIGHT/2)-myBall.size){
    myBall.angle = DEG2RAD(180)-myBall.angle;
  }

  if (myBall.y < -(SCRHEIGHT/2)){
    myBall.angle = DEG2RAD(180)-myBall.angle;
  }

  // Paddle collisions

  if ( (myBall.x < ( (-SCRWIDTH/2) + (myPaddle[0].width*2) )) && 
       ( (myBall.y + myBall.size) > myPaddle[0].y) && 
       (myBall.y < (myPaddle[0].y + myPaddle[0].height) ) )
						myBall.angle = DEG2RAD(360)-myBall.angle;

  if ( (myBall.x + myBall.size > ( (SCRWIDTH/2) - (myPaddle[1].width*2) )) && 
       ( (myBall.y + myBall.size) > myPaddle[1].y) && 
       (myBall.y < (myPaddle[1].y + myPaddle[1].height) ) )
						myBall.angle = DEG2RAD(360)-myBall.angle;

  // Scored!

  if ( (myBall.x < (-SCRWIDTH/2)) || (myBall.x + myBall.size >= ( SCRWIDTH/2) )){
    myBall.x = 0;
  }

  myBall.x+=myBall.speed*(sin(myBall.angle));
  myBall.y+=myBall.speed*(cos(myBall.angle));

  printf("myBall.x: %f\n", myBall.x);
  printf("myBall.y: %f\n", myBall.y);

  drawCuboid(myBall.x,myBall.y,0, myBall.size,myBall.size,0, ballColor);

}

void drawBackground(){
  aRGBAData aBKGColor[]={
    {0, 0, 255, 0},
    {0, 0, 255, 0},
    {0, 0, 255, 0},
    {0, 0, 255, 0}
  };

  drawCuboid(-SCRWIDTH/2, -SCRHEIGHT/2, -1, SCRWIDTH, SCRHEIGHT, 0, aBKGColor);
}

void drawPaddle(){
  int j;

  for (j=0; j<2; j++){
    if (myPaddle[j].y > (SCRHEIGHT/2)-myPaddle[j].height)
      myPaddle[j].y = (SCRHEIGHT/2)-myPaddle[j].height;

    if (myPaddle[j].y < -SCRHEIGHT/2)
      myPaddle[j].y = -SCRHEIGHT/2;
  }

  drawCuboid(-SCRWIDTH/2+myPaddle[0].width, myPaddle[0].y, 0, myPaddle[0].width, myPaddle[0].height, 0, NULL);

  drawCuboid(SCRWIDTH/2-(myPaddle[1].width*2), myPaddle[1].y, 0, myPaddle[1].width, myPaddle[1].height, 0, NULL);
}










